capture program drop panel_within
program panel_within
     syntax ,  epsi_m(varlist) explambda(namelist) expsigma(namelist) id(varlist)

     mata: epsilon = st_data(., "`epsi_m'")
     mata: lambda = st_numscalar("`explambda'")
     mata: sigma = st_numscalar("`expsigma'")

     mata: id = st_data(., "`id'")

     mata: lf_within(epsilon,  lambda, sigma, id)
     matrix ln_g = r(lng)

end

mata:
void lf_within(real vector epsi, real scalar lambda, real scalar sigma, numeric vector id)
{

     info=panelsetup(id,1) /* nX2; n is the number of firms; the "matrix" of id, the 1st column */
     n = rows(info) /* # of firms */
     s = J(n,1,.)
     lnMCDF1 = J(n,1,.)
     MCDF1 = J(n,1,.)
     lnMCDF3 = J(n,1,.)
     MCDF3 = J(n,1,.)
     lnMCDF3ghk = J(n,1,.)
     ln_h = J(n,1,.)
     lnmpdf = J(n,1,.)
     lng = 0

      /* external S */
      /* ghkfast_init_pivot(S,1) */

      for(i=1; i<=n;i++)
      {

          epsi_i = panelsubmatrix(epsi, i, info)
          s[i] = rows(epsi_i) /* ith panel's # of obs */
          t = s[i]

          A1 = t*I(t-1,t-1)-J(t-1,t-1,1)
          A1 = (1/t)*(A1,J(t-1,1,-1))

   /*       epsi_idmean = epsi_i - J(t,1,sum(epsi_i))  */

/* ============================      part 1      =============================================== */

/*          SIGMA1 = (1+lambda^2)*I(t,t)

          SIGMA1 = (SIGMA1 + SIGMA1')/2

          X1 = J(t,1,0)  */

/*          MCDF1[i] = ghkfast(S, X1', SIGMA1) */
/*          lnMCDF1[i] = ln(MCDF1[i])  */
/* lnMCDF1[i] = gausshermite_mata(X1',(SIGMA1[1,1]-SIGMA1[1,2])^0.5, SIGMA1[1,2]^0.5) */
lnMCDF1[i] = ln(1/(2^t))

/* ============================      part 2      =============================================== */
          SIGMA2 = sigma^2*(I(t-1,t-1) - 1/t*J(t-1,t-1,1))
          SIGMA2 = (SIGMA2 + SIGMA2')/2

          lnmpdf[i] = lnmvnpdf(epsi_i[1..(t-1)], SIGMA2)

/* ============================      part 3      =============================================== */

          SIGMA3 = I(t,t) + J(t,t,lambda^2/t)

          SIGMA3 = (SIGMA3 + SIGMA3')/2

/*            X3 = -lambda/sigma*(I(t,t)-1/t*J(t,t,1))*epsi_i   */
          X3 = -lambda/sigma*A1'*luinv(A1*A1')*epsi_i[1..t-1]

/*          MCDF3[i] = ghkfast(S, X3', SIGMA3)
          lnMCDF3[i] = ln(MCDF3[i])  */

 lnMCDF3[i] = gausshermite_mata(X3',(sqrt(SIGMA3[1,1]-SIGMA3[1,2])), sqrt(SIGMA3[1,2]))

/* =============================================================================================*/

          ln_h[i] = -lnMCDF1[i] + lnmpdf[i] + lnMCDF3[i]

          lng = lng + ln_h[i]

          }

     ln_g = J(s[1],1,ln_h[1])

     if (n>1)
     {
          for(i=2; i<=n; i++)
          {
               ln_g = (ln_g\J(s[i],1,ln_h[i]))
          }

          st_matrix("r(lng)", ln_g)

     }


     st_matrix("r(lng)", ln_g)                    

}

end


mata:
real scalar lnmvnpdf(real matrix x, real matrix SIGMA)
{

     k = length(x)

     lnf = -k/2*ln(2*pi())-1/2*ln(det(SIGMA))-1/2*x'*luinv(SIGMA)*x

     return(lnf)

}
end


mata:
real scalar gausshermite_mata(real matrix upper, real scalar s_d, real scalar s_c)
{
     t = length(upper)
     external noeuds_m
     external poids_m
     external cham_nd
     real x , g , f , fsum
     fsum = 0

     lncdf = J(t,cham_nd,.)
     lng = J(1,cham_nd,.)
     lnf = J(1,cham_nd,.)

     for(i=1; i<=cham_nd; i++)
     {
         x = noeuds_m[i]*(2^0.5)

         for(j=1; j<=t; j++)
         {
              lncdf[j,i] = lnnormal((upper[j]-s_c*x)/s_d)
         }
     }

     for(i=1; i<=cham_nd; i++)
     {
          lng[1,i] = 0

          for(j=1; j<=t; j++)
          {
               lng[1,i] = lng[1,i]+lncdf[j,i]
          }

          lnf[1,i] = ln(poids_m[i])-0.5*ln(pi())+lng[1,i]
     }

     maxlnf = max(lnf)

     for(i=1; i<=cham_nd; i++)
     {
          fsum = fsum + exp(lnf[1,i] - maxlnf)
     }

     if(fsum == 0)
     {
          "fsum = "
          fsum
     }

     lnfsum = ln(fsum) + maxlnf

     return(lnfsum)

}
end
